var Promise = require('es6-promise-polyfill').Promise;
var exec = require('child_process').exec;
var number = require('./utils').number;
var quote = require('./utils').quote;
var extractFilename = require('./utils').extractFilename;
var append = require('./utils').append;

function makeArguments(filename, settings) {
  var info = extractFilename(filename);
  var pattern = settings.pattern || '';
  var values = {
    line: info.line + number(settings.line, 1),
    column: info.column + number(settings.column, 1)
  };

  if (!/\{filename\}/.test(pattern)) {
    pattern = append(pattern, '{filename}:{line}:{column}');
  }

  return pattern
    .replace(
      new RegExp('\\{(' + Object.keys(values).join('|') + ')\\}', 'g'),
      function(m, name) {
        return values[name];
      }
    )
    // replace `{filename}` and joined right string for quoted filename,
    // as it could has spaces
    //
    //   {filename} --line 1 --column 2
    //   => "filename" --line 1 --column 2
    //
    //   {filename}:1:2
    //   => "filename:1:2"
    //
    .replace(/\{filename\}(\S*)/, function(m, rest) {
      return quote(info.filename + rest);
    });
}

function open(cmd, filename, settings) {
  return new Promise(function(resolve, reject) {
    cmd = append(quote(cmd), makeArguments(filename, settings || {}));

    exec(cmd, function(err) {
      if (err) {
        reject(err);
      } else {
        resolve();
      }
    });
  });
}

module.exports = open;
module.exports.factory = function(cmd) {
  return function openInEditor(filename, settings) {
    return open(cmd, filename, settings);
  };
};

module.exports.detectAndOpenFactory = function(detect, settings) {
  return function openInEditor(filename) {
    return detect().then(function(cmd) {
      open(cmd, filename, settings);
    });
  };
};
